@@ -64,7 +64,7 @@ void Archive::CheckArc(bool EnableBroken)
 
 
 #if !defined(SHELL_EXT) && !defined(SFX_MODULE)
-void Archive::CheckOpen(char *Name,wchar *NameW)
+void Archive::CheckOpen(const char *Name,const wchar *NameW)
 {
   TOpen(Name,NameW);
   CheckArc(false);
@@ -72,7 +72,7 @@ void Archive::CheckOpen(char *Name,wchar *NameW)
 #endif
 
 
-bool Archive::WCheckOpen(char *Name,wchar *NameW)
+bool Archive::WCheckOpen(const char *Name,const wchar *NameW)
 {
   if (!WOpen(Name,NameW))
     return(false);
@@ -132,7 +132,7 @@ bool Archive::IsArchive(bool EnableBroken)
   else
   {
     Array<char> Buffer(MAXSFXSIZE);
-    long CurPos=int64to32(Tell());
+    long CurPos=(long)Tell();
     int ReadSize=Read(&Buffer[0],Buffer.Size()-16);
     for (int I=0;I<ReadSize;I++)
       if (Buffer[I]==0x52 && IsSignature((byte *)&Buffer[I]))
@@ -204,10 +204,10 @@ bool Archive::IsArchive(bool EnableBroken)
   if (!SilentOpen || !Encrypted)
   {
     SaveFilePos SavePos(*this);
-    Int64 SaveCurBlockPos=CurBlockPos,SaveNextBlockPos=NextBlockPos;
+    int64 SaveCurBlockPos=CurBlockPos,SaveNextBlockPos=NextBlockPos;
 
     NotFirstVolume=false;
-    while (ReadHeader())
+    while (ReadHeader()!=0)
     {
       int HeaderType=GetHeaderType();
       if (HeaderType==NEWSUB_HEAD)
@@ -233,7 +233,7 @@ bool Archive::IsArchive(bool EnableBroken)
   if (!Volume || !NotFirstVolume)
   {
     strcpy(FirstVolumeName,FileName);
-    strcpyw(FirstVolumeNameW,FileNameW);
+    wcscpy(FirstVolumeNameW,FileNameW);
   }
 
   return(true);
