@@ -21,25 +21,24 @@ void CmdExtract::DoExtract(CommandData *Cmd)
   PasswordCancelled=false;
   DataIO.SetCurrentCommand(*Cmd->Command);
 
-  struct FindData FD;
-  while (Cmd->GetArcName(ArcName,ArcNameW,sizeof(ArcName)))
+  FindData FD;
+  while (Cmd->GetArcName(ArcName,ArcNameW,ASIZE(ArcName)))
     if (FindFile::FastFind(ArcName,ArcNameW,&FD))
       DataIO.TotalArcSize+=FD.Size;
+
   Cmd->ArcNames->Rewind();
-  while (Cmd->GetArcName(ArcName,ArcNameW,sizeof(ArcName)))
+  while (Cmd->GetArcName(ArcName,ArcNameW,ASIZE(ArcName)))
   {
     while (true)
     {
-      char PrevCmdPassword[MAXPASSWORD];
-      strcpy(PrevCmdPassword,Cmd->Password);
+      wchar PrevCmdPassword[MAXPASSWORD];
+      wcscpy(PrevCmdPassword,Cmd->Password);
 
       EXTRACT_ARC_CODE Code=ExtractArchive(Cmd);
 
-/*
-      restore Cmd->Password which could be changed in IsArchive() call
-      for next header encrypted archive
-*/
-      strcpy(Cmd->Password,PrevCmdPassword);
+      // Restore Cmd->Password, which could be changed in IsArchive() call
+      // for next header encrypted archive.
+      wcscpy(Cmd->Password,PrevCmdPassword);
 
       if (Code!=EXTRACT_ARC_REPEAT)
         break;
@@ -54,7 +53,7 @@ void CmdExtract::DoExtract(CommandData *Cmd)
     {
       mprintf(St(MExtrNoFiles));
     }
-    ErrHandler.SetErrorCode(WARNING);
+    ErrHandler.SetErrorCode(NO_FILES_ERROR);
   }
 #ifndef GUI
   else
@@ -81,7 +80,7 @@ void CmdExtract::ExtractArchiveInit(CommandData *Cmd,Archive &Arc)
 #endif
 
   if (*Cmd->Password!=0)
-    strcpy(Password,Cmd->Password);
+    wcscpy(Password,Cmd->Password);
   PasswordAll=(*Cmd->Password!=0);
 
   DataIO.UnpVolume=false;
@@ -90,6 +89,9 @@ void CmdExtract::ExtractArchiveInit(CommandData *Cmd,Archive &Arc)
   SignatureFound=false;
   AllMatchesExact=true;
   ReconstructDone=false;
+  AnySolidDataUnpackedWell=false;
+
+  StartTime.SetCurrentTime();
 }
 
 
@@ -112,6 +114,7 @@ EXTRACT_ARC_CODE CmdExtract::ExtractArchive(CommandData *Cmd)
     return(EXTRACT_ARC_NEXT);
   }
 
+  // archive with corrupt encrypted header can be closed in IsArchive() call
   if (!Arc.IsOpened())
     return(EXTRACT_ARC_NEXT);
 
@@ -119,13 +122,45 @@ EXTRACT_ARC_CODE CmdExtract::ExtractArchive(CommandData *Cmd)
   if (Arc.Volume && Arc.NotFirstVolume)
   {
     char FirstVolName[NM];
+    VolNameToFirstName(ArcName,FirstVolName,(Arc.NewMhd.Flags & MHD_NEWNUMBERING)!=0);
 
-    VolNameToFirstName(ArcName,FirstVolName,(Arc.NewMhd.Flags & MHD_NEWNUMBERING));
+    // If several volume names from same volume set are specified
+    // and current volume is not first in set and first volume is present
+    // and specified too, let's skip the current volume.
     if (stricomp(ArcName,FirstVolName)!=0 && FileExist(FirstVolName) &&
         Cmd->ArcNames->Search(FirstVolName,NULL,false))
       return(EXTRACT_ARC_NEXT);
   }
 #endif
+
+  int64 VolumeSetSize=0; // Total size of volumes after the current volume.
+
+  if (Arc.Volume)
+  {
+    // Calculate the total size of all accessible volumes.
+    // This size is necessary to display the correct total progress indicator.
+
+    char NextName[NM];
+    wchar NextNameW[NM];
+
+    strcpy(NextName,Arc.FileName);
+    wcscpy(NextNameW,Arc.FileNameW);
+
+    while (true)
+    {
+      // First volume is already added to DataIO.TotalArcSize 
+      // in initial TotalArcSize calculation in DoExtract.
+      // So we skip it and start from second volume.
+      NextVolumeName(NextName,NextNameW,ASIZE(NextName),(Arc.NewMhd.Flags & MHD_NEWNUMBERING)==0 || Arc.OldFormat);
+      struct FindData FD;
+      if (FindFile::FastFind(NextName,NextNameW,&FD))
+        VolumeSetSize+=FD.Size;
+      else
+        break;
+    }
+    DataIO.TotalArcSize+=VolumeSetSize;
+  }
+
   ExtractArchiveInit(Cmd,Arc);
 
   if (*Cmd->Command=='T' || *Cmd->Command=='I')
@@ -150,11 +185,20 @@ EXTRACT_ARC_CODE CmdExtract::ExtractArchive(CommandData *Cmd)
 
   while (1)
   {
-    int Size=Arc.ReadHeader();
+    size_t Size=Arc.ReadHeader();
     bool Repeat=false;
     if (!ExtractCurrentFile(Cmd,Arc,Size,Repeat))
       if (Repeat)
       {
+        // If we started extraction from not first volume and need to
+        // restart it from first, we must correct DataIO.TotalArcSize
+        // for correct total progress display. We subtract the size
+        // of current volume and all volumes after it and add the size
+        // of new (first) volume.
+        FindData OldArc,NewArc;
+        if (FindFile::FastFind(Arc.FileName,Arc.FileNameW,&OldArc) &&
+            FindFile::FastFind(ArcName,ArcNameW,&NewArc))
+          DataIO.TotalArcSize-=VolumeSetSize+OldArc.Size-NewArc.Size;
         return(EXTRACT_ARC_REPEAT);
       }
       else
@@ -165,16 +209,16 @@ EXTRACT_ARC_CODE CmdExtract::ExtractArchive(CommandData *Cmd)
 }
 
 
-bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize,bool &Repeat)
+bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,size_t HeaderSize,bool &Repeat)
 {
   char Command=*Cmd->Command;
-  if (HeaderSize<=0)
+  if (HeaderSize==0)
     if (DataIO.UnpVolume)
     {
 #ifdef NOVOLUME
       return(false);
 #else
-      if (!MergeArchive(Arc,NULL,false,Command))
+      if (!MergeArchive(Arc,&DataIO,false,Command))
       {
         ErrHandler.SetErrorCode(WARNING);
         return(false);
@@ -206,7 +250,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
       if (Arc.EndArcHead.Flags & EARC_NEXT_VOLUME)
       {
 #ifndef NOVOLUME
-        if (!MergeArchive(Arc,NULL,false,Command))
+        if (!MergeArchive(Arc,&DataIO,false,Command))
         {
           ErrHandler.SetErrorCode(WARNING);
           return(false);
@@ -306,15 +350,20 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
   Arc.ConvertAttributes();
 
 #ifndef SFX_MODULE
-  if ((Arc.NewLhd.Flags & (LHD_SPLIT_BEFORE/*|LHD_SOLID*/)) && FirstFile)
+  if ((Arc.NewLhd.Flags & LHD_SPLIT_BEFORE) && FirstFile)
   {
     char CurVolName[NM];
     strcpy(CurVolName,ArcName);
 
-    VolNameToFirstName(ArcName,ArcName,(Arc.NewMhd.Flags & MHD_NEWNUMBERING));
-    if (stricomp(ArcName,CurVolName)!=0 && FileExist(ArcName))
+    bool NewNumbering=(Arc.NewMhd.Flags & MHD_NEWNUMBERING)!=0;
+    VolNameToFirstName(ArcName,ArcName,NewNumbering);
+    if (*ArcNameW!=0)
+      VolNameToFirstName(ArcNameW,ArcNameW,NewNumbering);
+
+    if (stricomp(ArcName,CurVolName)!=0 && FileExist(ArcName,ArcNameW))
     {
-      *ArcNameW=0;
+      // If first volume name does not match the current name and if
+      // such volume name really exists, let's unpack from this first volume.
       Repeat=true;
       return(false);
     }
@@ -334,7 +383,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
     strcpy(ArcName,CurVolName);
   }
 #endif
-  DataIO.UnpVolume=(Arc.NewLhd.Flags & LHD_SPLIT_AFTER);
+  DataIO.UnpVolume=(Arc.NewLhd.Flags & LHD_SPLIT_AFTER)!=0;
   DataIO.NextVolumeMissing=false;
 
   Arc.Seek(Arc.NextBlockPos-Arc.NewLhd.FullPackSize,SEEK_SET);
@@ -352,7 +401,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
 #ifdef RARDLL
       Cmd->DllError=ERAR_BAD_DATA;
 #endif
-      ErrHandler.SetErrorCode(WARNING);
+      ErrHandler.SetErrorCode(OPEN_ERROR);
     }
     ExactMatch=false;
   }
@@ -369,13 +418,18 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
       {
 #ifdef RARDLL
         if (*Cmd->Password==0)
+        {
+          char PasswordA[MAXPASSWORD];
+
           if (Cmd->Callback==NULL ||
-              Cmd->Callback(UCM_NEEDPASSWORD,Cmd->UserData,(LONG)Cmd->Password,sizeof(Cmd->Password))==-1)
+              Cmd->Callback(UCM_NEEDPASSWORD,Cmd->UserData,(LPARAM)PasswordA,ASIZE(PasswordA))==-1)
             return(false);
-        strcpy(Password,Cmd->Password);
+          GetWideName(PasswordA,NULL,Cmd->Password,ASIZE(Cmd->Password));
+        }
+        wcscpy(Password,Cmd->Password);
 
 #else
-        if (!GetPassword(PASSWORD_FILE,ArcFileName,Password,sizeof(Password)))
+        if (!GetPassword(PASSWORD_FILE,ArcFileName,ArcFileNameW,Password,ASIZE(Password)))
         {
           PasswordCancelled=true;
           return(false);
@@ -392,7 +446,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
             case -1:
               ErrHandler.Exit(USER_BREAK);
             case 2:
-              if (!GetPassword(PASSWORD_FILE,ArcFileName,Password,sizeof(Password)))
+              if (!GetPassword(PASSWORD_FILE,ArcFileName,ArcFileNameW,Password,ASIZE(Password)))
               {
                 return(false);
               }
@@ -425,7 +479,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
 
     bool EmptyName=false;
 #ifndef SFX_MODULE
-    int Length=strlen(Cmd->ArcPath);
+    size_t Length=strlen(Cmd->ArcPath);
     if (Length>1 && IsPathDiv(Cmd->ArcPath[Length-1]) &&
         strlen(ArcFileName)==Length-1)
       Length--;
@@ -466,7 +520,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
     if (WideName)
     {
       if (*Cmd->ExtrPathW!=0)
-        strcpyw(DestFileNameW,Cmd->ExtrPathW);
+        wcscpy(DestFileNameW,Cmd->ExtrPathW);
       else
         CharToWide(Cmd->ExtrPath,DestFileNameW);
 
@@ -475,10 +529,10 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
       {
         wchar FileNameW[NM];
         if (*Arc.FirstVolumeNameW!=0)
-          strcpyw(FileNameW,Arc.FirstVolumeNameW);
+          wcscpy(FileNameW,Arc.FirstVolumeNameW);
         else
           CharToWide(Arc.FirstVolumeName,FileNameW);
-        strcatw(DestFileNameW,PointToName(FileNameW));
+        wcscat(DestFileNameW,PointToName(FileNameW));
         SetExt(DestFileNameW,NULL);
         AddEndSlash(DestFileNameW);
       }
@@ -488,8 +542,8 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
       if (Length>0)
       {
         wchar ArcPathW[NM];
-        CharToWide(Cmd->ArcPath,ArcPathW);
-        Length=strlenw(ArcPathW);
+        GetWideName(Cmd->ArcPath,Cmd->ArcPathW,ArcPathW,ASIZE(ArcPathW));
+        Length=wcslen(ArcPathW);
       }
       ExtrNameW+=Length;
       while (*ExtrNameW==CPATHDIVIDER)
@@ -500,9 +554,9 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
         *DestFileNameW=0;
 
       if (Command=='E' || Cmd->ExclPath==EXCL_SKIPWHOLEPATH)
-        strcatw(DestFileNameW,PointToName(ExtrNameW));
+        wcscat(DestFileNameW,PointToName(ExtrNameW));
       else
-        strcatw(DestFileNameW,ExtrNameW);
+        wcscat(DestFileNameW,ExtrNameW);
 
       if (AbsPaths && DestFileNameW[1]=='_' && IsPathDiv(DestFileNameW[2]))
         DestFileNameW[1]=':';
@@ -518,14 +572,23 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
       if (FindFile::FastFind(DestFileName,DestNameW,&FD))
       {
         if (FD.mtime >= Arc.NewLhd.mtime)
-          ExtrFile=false;
+        {
+          // If directory already exists and its modification time is newer 
+          // than start of extraction, it is likely it was created 
+          // when creating a path to one of already extracted items. 
+          // In such case we'll better update its time even if archived 
+          // directory is older.
+
+          if (!FD.IsDir || FD.mtime<StartTime)
+            ExtrFile=false;
+        }
       }
       else
         if (Cmd->FreshFiles)
           ExtrFile=false;
     }
 
-    // skip encrypted file if no password is specified
+    // Skip encrypted file if no password is specified.
     if ((Arc.NewLhd.Flags & LHD_PASSWORD)!=0 && *Password==0)
     {
       ErrHandler.SetErrorCode(WARNING);
@@ -545,7 +608,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
     }
     if (*Cmd->DllDestNameW)
     {
-      strncpyzw(DestFileNameW,Cmd->DllDestNameW,ASIZE(DestFileNameW));
+      wcsncpyz(DestFileNameW,Cmd->DllDestNameW,ASIZE(DestFileNameW));
       DestNameW=DestFileNameW;
       if (Cmd->DllOpMode!=RAR_EXTRACT)
         ExtrFile=false;
@@ -595,7 +658,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
 #endif
           return(true);
         }
-        MKDIR_CODE MDCode=MakeDir(DestFileName,DestNameW,Arc.NewLhd.FileAttr);
+        MKDIR_CODE MDCode=MakeDir(DestFileName,DestNameW,!Cmd->IgnoreGeneralAttr,Arc.NewLhd.FileAttr);
         bool DirExist=false;
         if (MDCode!=MKDIR_SUCCESS)
         {
@@ -607,7 +670,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
             DirExist=false;
           }
           CreatePath(DestFileName,DestNameW,true);
-          MDCode=MakeDir(DestFileName,DestNameW,Arc.NewLhd.FileAttr);
+          MDCode=MakeDir(DestFileName,DestNameW,!Cmd->IgnoreGeneralAttr,Arc.NewLhd.FileAttr);
         }
         if (MDCode==MKDIR_SUCCESS)
         {
@@ -620,7 +683,8 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
         else
           if (DirExist)
           {
-            SetFileAttr(DestFileName,DestNameW,Arc.NewLhd.FileAttr);
+            if (!Cmd->IgnoreGeneralAttr)
+              SetFileAttr(DestFileName,DestNameW,Arc.NewLhd.FileAttr);
             PrevExtracted=true;
           }
           else
@@ -634,12 +698,12 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
           }
         if (PrevExtracted)
         {
-#if defined(_WIN_32) && !defined(_WIN_CE) && !defined(SFX_MODULE)
+#if defined(_WIN_ALL) && !defined(_WIN_CE) && !defined(SFX_MODULE)
           if (Cmd->SetCompressedAttr &&
               (Arc.NewLhd.FileAttr & FILE_ATTRIBUTE_COMPRESSED)!=0 && WinNT())
-            SetFileCompression(DestFileName,DestFileNameW,true);
+            SetFileCompression(DestFileName,DestNameW,true);
 #endif
-          SetDirTime(DestFileName,
+          SetDirTime(DestFileName,DestNameW,
             Cmd->xmtime==EXTTIME_NONE ? NULL:&Arc.NewLhd.mtime,
             Cmd->xctime==EXTTIME_NONE ? NULL:&Arc.NewLhd.ctime,
             Cmd->xatime==EXTTIME_NONE ? NULL:&Arc.NewLhd.atime);
@@ -662,7 +726,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
             ExtrFile=false;
             if (!UserReject)
             {
-              ErrHandler.CreateErrorMsg(Arc.FileName,DestFileName);
+              ErrHandler.CreateErrorMsg(Arc.FileName,Arc.FileNameW,DestFileName,DestFileNameW);
               ErrHandler.SetErrorCode(CREATE_ERROR);
 #ifdef RARDLL
               Cmd->DllError=ERAR_ECREATE;
@@ -683,7 +747,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
                   ExtrFile=true;
                 }
                 else
-                  ErrHandler.CreateErrorMsg(Arc.FileName,DestFileName);
+                  ErrHandler.CreateErrorMsg(Arc.FileName,Arc.FileNameW,DestFileName,DestFileNameW);
               }
             }
           }
@@ -695,6 +759,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
       SkipSolid=true;
       TestMode=true;
       ExtrFile=true;
+
     }
     if (ExtrFile)
     {
@@ -703,7 +768,7 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
         if (!TestMode && Command!='P' && CurFile.IsDevice())
         {
           Log(Arc.FileName,St(MInvalidName),DestFileName);
-          ErrHandler.WriteError(Arc.FileName,DestFileName);
+          ErrHandler.WriteError(Arc.FileName,Arc.FileNameW,DestFileName,DestFileNameW);
         }
         TotalFileCount++;
       }
@@ -735,9 +800,25 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
       DataIO.CurUnpWrite=0;
       DataIO.UnpFileCRC=Arc.OldFormat ? 0 : 0xffffffff;
       DataIO.PackedCRC=0xffffffff;
+
+      wchar FilePassword[MAXPASSWORD];
+#ifdef _WIN_ALL
+      if (Arc.NewLhd.HostOS==HOST_MSDOS/* && Arc.NewLhd.UnpVer<=25*/)
+      {
+        // We need the password in OEM encoding if file was encrypted by
+        // native RAR/DOS (not extender based). Let's make the conversion.
+        char PswA[MAXPASSWORD];
+        CharToOemBuffW(Password,PswA,ASIZE(PswA));
+        CharToWide(PswA,FilePassword,ASIZE(FilePassword));
+        FilePassword[ASIZE(FilePassword)-1]=0;
+      }
+      else
+#endif
+        wcscpy(FilePassword,Password);
+      
       DataIO.SetEncryption(
-        (Arc.NewLhd.Flags & LHD_PASSWORD) ? Arc.NewLhd.UnpVer:0,Password,
-        (Arc.NewLhd.Flags & LHD_SALT) ? Arc.NewLhd.Salt:NULL,false,
+        (Arc.NewLhd.Flags & LHD_PASSWORD)!=0 ? Arc.NewLhd.UnpVer:0,FilePassword,
+        (Arc.NewLhd.Flags & LHD_SALT)!=0 ? Arc.NewLhd.Salt:NULL,false,
         Arc.NewLhd.UnpVer>=36);
       DataIO.SetPackedSizeToRead(Arc.NewLhd.FullPackSize);
       DataIO.SetFiles(&Arc,&CurFile);
@@ -767,17 +848,29 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
               Unp->DoUnpack(15,FileCount>1 && Arc.Solid);
             else
 #endif
-              Unp->DoUnpack(Arc.NewLhd.UnpVer,Arc.NewLhd.Flags & LHD_SOLID);
+              Unp->DoUnpack(Arc.NewLhd.UnpVer,(Arc.NewLhd.Flags & LHD_SOLID)!=0);
           }
 
       if (Arc.IsOpened())
         Arc.SeekToNext();
 
+      bool ValidCRC=Arc.OldFormat && UINT32(DataIO.UnpFileCRC)==UINT32(Arc.NewLhd.FileCRC) ||
+                   !Arc.OldFormat && UINT32(DataIO.UnpFileCRC)==UINT32(Arc.NewLhd.FileCRC^0xffffffff);
+
+      // We set AnySolidDataUnpackedWell to true if we found at least one
+      // valid non-zero solid file in preceding solid stream. If it is true
+      // and if current encrypted file is broken, we do not need to hint
+      // about a wrong password and can report CRC error only.
+      if ((Arc.NewLhd.Flags & LHD_SOLID)==0)
+        AnySolidDataUnpackedWell=false; // Reset the flag, because non-solid file is found.
+      else
+        if (Arc.NewLhd.Method!=0x30 && Arc.NewLhd.FullUnpSize>0 && ValidCRC)
+          AnySolidDataUnpackedWell=true;
+ 
       bool BrokenFile=false;
       if (!SkipSolid)
       {
-        if (Arc.OldFormat && UINT32(DataIO.UnpFileCRC)==UINT32(Arc.NewLhd.FileCRC) ||
-            !Arc.OldFormat && UINT32(DataIO.UnpFileCRC)==UINT32(Arc.NewLhd.FileCRC^0xffffffff))
+        if (ValidCRC)
         {
 #ifndef GUI
           if (Command!='P' && Command!='I')
@@ -786,14 +879,13 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
         }
         else
         {
-          char *BadArcName=/*(Arc.NewLhd.Flags & LHD_SPLIT_BEFORE) ? NULL:*/Arc.FileName;
-          if (Arc.NewLhd.Flags & LHD_PASSWORD)
+          if ((Arc.NewLhd.Flags & LHD_PASSWORD)!=0 && !AnySolidDataUnpackedWell)
           {
-            Log(BadArcName,St(MEncrBadCRC),ArcFileName);
+            Log(Arc.FileName,St(MEncrBadCRC),ArcFileName);
           }
           else
           {
-            Log(BadArcName,St(MCRCFailed),ArcFileName);
+            Log(Arc.FileName,St(MCRCFailed),ArcFileName);
           }
           BrokenFile=true;
           ErrHandler.SetErrorCode(CRC_ERROR);
@@ -811,32 +903,29 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
       if (!TestMode && (Command=='X' || Command=='E') &&
           !IsLink(Arc.NewLhd.FileAttr))
       {
-#if defined(_WIN_32) || defined(_EMX)
+#if defined(_WIN_ALL) || defined(_EMX)
         if (Cmd->ClearArc)
           Arc.NewLhd.FileAttr&=~FA_ARCH;
-/*
-        else
-          Arc.NewLhd.FileAttr|=FA_ARCH; //set archive bit for unpacked files (file is not backed up)
-*/
 #endif
         if (!BrokenFile || Cmd->KeepBroken)
         {
           if (BrokenFile)
             CurFile.Truncate();
-          CurFile.SetOpenFileStat(
+          CurFile.SetOpenFileTime(
             Cmd->xmtime==EXTTIME_NONE ? NULL:&Arc.NewLhd.mtime,
             Cmd->xctime==EXTTIME_NONE ? NULL:&Arc.NewLhd.ctime,
             Cmd->xatime==EXTTIME_NONE ? NULL:&Arc.NewLhd.atime);
           CurFile.Close();
-#if defined(_WIN_32) && !defined(_WIN_CE) && !defined(SFX_MODULE)
+#if defined(_WIN_ALL) && !defined(_WIN_CE) && !defined(SFX_MODULE)
           if (Cmd->SetCompressedAttr &&
               (Arc.NewLhd.FileAttr & FILE_ATTRIBUTE_COMPRESSED)!=0 && WinNT())
             SetFileCompression(CurFile.FileName,CurFile.FileNameW,true);
 #endif
-          CurFile.SetCloseFileStat(
+          CurFile.SetCloseFileTime(
             Cmd->xmtime==EXTTIME_NONE ? NULL:&Arc.NewLhd.mtime,
-            Cmd->xatime==EXTTIME_NONE ? NULL:&Arc.NewLhd.atime,
-            Arc.NewLhd.FileAttr);
+            Cmd->xatime==EXTTIME_NONE ? NULL:&Arc.NewLhd.atime);
+          if (!Cmd->IgnoreGeneralAttr)
+            SetFileAttr(CurFile.FileName,CurFile.FileNameW,Arc.NewLhd.FileAttr);
           PrevExtracted=true;
         }
       }
@@ -856,15 +945,15 @@ bool CmdExtract::ExtractCurrentFile(CommandData *Cmd,Archive &Arc,int HeaderSize
 }
 
 
-void CmdExtract::UnstoreFile(ComprDataIO &DataIO,Int64 DestUnpSize)
+void CmdExtract::UnstoreFile(ComprDataIO &DataIO,int64 DestUnpSize)
 {
   Array<byte> Buffer(0x10000);
   while (1)
   {
-    unsigned int Code=DataIO.UnpRead(&Buffer[0],Buffer.Size());
+    uint Code=DataIO.UnpRead(&Buffer[0],Buffer.Size());
     if (Code==0 || (int)Code==-1)
       break;
-    Code=Code<DestUnpSize ? Code:int64to32(DestUnpSize);
+    Code=Code<DestUnpSize ? Code:(uint)DestUnpSize;
     DataIO.UnpWrite(&Buffer[0],Code);
     if (DestUnpSize>=0)
       DestUnpSize-=Code;
