@@ -18,7 +18,7 @@ File::File()
   AllowDelete=true;
   CloseCount=0;
   AllowExceptions=true;
-#ifdef _WIN_32
+#ifdef _WIN_ALL
   NoSequentialRead=false;
 #endif
 }
@@ -51,7 +51,7 @@ bool File::Open(const char *Name,const wchar *NameW,bool OpenShared,bool Update)
   FileHandle hNewFile;
   if (File::OpenShared)
     OpenShared=true;
-#ifdef _WIN_32
+#ifdef _WIN_ALL
   uint Access=GENERIC_READ;
   if (Update)
     Access|=GENERIC_WRITE;
@@ -62,7 +62,7 @@ bool File::Open(const char *Name,const wchar *NameW,bool OpenShared,bool Update)
   if (WinNT() && NameW!=NULL && *NameW!=0)
     hNewFile=CreateFileW(NameW,Access,ShareMode,NULL,OPEN_EXISTING,Flags,NULL);
   else
-    hNewFile=CreateFile(Name,Access,ShareMode,NULL,OPEN_EXISTING,Flags,NULL);
+    hNewFile=CreateFileA(Name,Access,ShareMode,NULL,OPEN_EXISTING,Flags,NULL);
 
   if (hNewFile==BAD_HANDLE && GetLastError()==ERROR_FILE_NOT_FOUND)
     ErrorType=FILE_NOTFOUND;
@@ -104,7 +104,7 @@ bool File::Open(const char *Name,const wchar *NameW,bool OpenShared,bool Update)
   {
     hFile=hNewFile;
     if (NameW!=NULL)
-      strcpyw(FileNameW,NameW);
+      wcscpy(FileNameW,NameW);
     else
       *FileNameW=0;
     if (Name!=NULL)
@@ -130,20 +130,20 @@ bool File::WOpen(const char *Name,const wchar *NameW)
 {
   if (Open(Name,NameW))
     return(true);
-  ErrHandler.OpenErrorMsg(Name);
+  ErrHandler.OpenErrorMsg(Name,NameW);
   return(false);
 }
 
 
 bool File::Create(const char *Name,const wchar *NameW,bool ShareRead)
 {
-#ifdef _WIN_32
+#ifdef _WIN_ALL
   DWORD ShareMode=(ShareRead || File::OpenShared) ? FILE_SHARE_READ:0;
   if (WinNT() && NameW!=NULL && *NameW!=0)
     hFile=CreateFileW(NameW,GENERIC_READ|GENERIC_WRITE,ShareMode,NULL,
                       CREATE_ALWAYS,0,NULL);
   else
-    hFile=CreateFile(Name,GENERIC_READ|GENERIC_WRITE,ShareMode,NULL,
+    hFile=CreateFileA(Name,GENERIC_READ|GENERIC_WRITE,ShareMode,NULL,
                      CREATE_ALWAYS,0,NULL);
 #else
   hFile=fopen(Name,CREATEBINARY);
@@ -152,7 +152,7 @@ bool File::Create(const char *Name,const wchar *NameW,bool ShareRead)
   HandleType=FILE_HANDLENORMAL;
   SkipClose=false;
   if (NameW!=NULL)
-    strcpyw(FileNameW,NameW);
+    wcscpy(FileNameW,NameW);
   else
     *FileNameW=0;
   if (Name!=NULL)
@@ -190,7 +190,7 @@ bool File::WCreate(const char *Name,const wchar *NameW,bool ShareRead)
   if (Create(Name,NameW,ShareRead))
     return(true);
   ErrHandler.SetErrorCode(CREATE_ERROR);
-  ErrHandler.CreateErrorMsg(Name);
+  ErrHandler.CreateErrorMsg(Name,NameW);
   return(false);
 }
 
@@ -205,7 +205,7 @@ bool File::Close()
     {
       if (!SkipClose)
       {
-#ifdef _WIN_32
+#ifdef _WIN_ALL
         Success=CloseHandle(hFile)==TRUE;
 #else
         Success=fclose(hFile)!=EOF;
@@ -220,7 +220,7 @@ bool File::Close()
       }
       hFile=BAD_HANDLE;
       if (!Success && AllowExceptions)
-        ErrHandler.CloseError(FileName);
+        ErrHandler.CloseError(FileName,FileNameW);
     }
   CloseCount++;
   return(Success);
@@ -229,7 +229,7 @@ bool File::Close()
 
 void File::Flush()
 {
-#ifdef _WIN_32
+#ifdef _WIN_ALL
   FlushFileBuffers(hFile);
 #else
   fflush(hFile);
@@ -254,7 +254,7 @@ bool File::Rename(const char *NewName,const wchar *NewNameW)
   // we do not need to rename if names are already same
   bool Success=strcmp(FileName,NewName)==0;
   if (Success && *FileNameW!=0 && *NullToEmpty(NewNameW)!=0)
-    Success=strcmpw(FileNameW,NewNameW)==0;
+    Success=wcscmp(FileNameW,NewNameW)==0;
 
   if (!Success)
     Success=RenameFile(FileName,FileNameW,NewName,NewNameW);
@@ -263,7 +263,7 @@ bool File::Rename(const char *NewName,const wchar *NewNameW)
   {
     // renamed successfully, storing the new name
     strcpy(FileName,NewName);
-    strcpyw(FileNameW,NullToEmpty(NewNameW));
+    wcscpy(FileNameW,NullToEmpty(NewNameW));
   }
   return(Success);
 }
@@ -278,14 +278,14 @@ void File::Write(const void *Data,size_t Size)
     switch(HandleType)
     {
       case FILE_HANDLESTD:
-#ifdef _WIN_32
+#ifdef _WIN_ALL
         hFile=GetStdHandle(STD_OUTPUT_HANDLE);
 #else
         hFile=stdout;
 #endif
         break;
       case FILE_HANDLEERR:
-#ifdef _WIN_32
+#ifdef _WIN_ALL
         hFile=GetStdHandle(STD_ERROR_HANDLE);
 #else
         hFile=stderr;
@@ -296,18 +296,18 @@ void File::Write(const void *Data,size_t Size)
   while (1)
   {
     bool Success=false;
-#ifdef _WIN_32
+#ifdef _WIN_ALL
     DWORD Written=0;
     if (HandleType!=FILE_HANDLENORMAL)
     {
       // writing to stdout can fail in old Windows if data block is too large
       const size_t MaxSize=0x4000;
       for (size_t I=0;I<Size;I+=MaxSize)
-	  {
+      {
         Success=WriteFile(hFile,(byte *)Data+I,(DWORD)Min(Size-I,MaxSize),&Written,NULL)==TRUE;
         if (!Success)
           break;
-	  }
+      }
     }
     else
       Success=WriteFile(hFile,Data,(DWORD)Size,&Written,NULL)==TRUE;
@@ -317,24 +317,24 @@ void File::Write(const void *Data,size_t Size)
 #endif
     if (!Success && AllowExceptions && HandleType==FILE_HANDLENORMAL)
     {
-#if defined(_WIN_32) && !defined(SFX_MODULE) && !defined(RARDLL)
+#if defined(_WIN_ALL) && !defined(SFX_MODULE) && !defined(RARDLL)
       int ErrCode=GetLastError();
       int64 FilePos=Tell();
       uint64 FreeSize=GetFreeDisk(FileName);
       SetLastError(ErrCode);
       if (FreeSize>Size && FilePos-Size<=0xffffffff && FilePos+Size>0xffffffff)
-        ErrHandler.WriteErrorFAT(FileName);
+        ErrHandler.WriteErrorFAT(FileName,FileNameW);
 #endif
-      if (ErrHandler.AskRepeatWrite(FileName,false))
+      if (ErrHandler.AskRepeatWrite(FileName,FileNameW,false))
       {
-#ifndef _WIN_32
+#ifndef _WIN_ALL
         clearerr(hFile);
 #endif
         if (Written<Size && Written>0)
           Seek(Tell()-Written,SEEK_SET);
         continue;
       }
-      ErrHandler.WriteError(NULL,FileName);
+      ErrHandler.WriteError(NULL,NULL,FileName,FileNameW);
     }
     break;
   }
@@ -344,7 +344,7 @@ void File::Write(const void *Data,size_t Size)
 
 int File::Read(void *Data,size_t Size)
 {
-  int64 FilePos=0; //initialized only to suppress some compilers warning
+  int64 FilePos=0; // Initialized only to suppress some compilers warning.
 
   if (IgnoreReadErrors)
     FilePos=Tell();
@@ -369,9 +369,9 @@ int File::Read(void *Data,size_t Size)
         }
         else
         {
-          if (HandleType==FILE_HANDLENORMAL && ErrHandler.AskRepeatRead(FileName))
+          if (HandleType==FILE_HANDLENORMAL && ErrHandler.AskRepeatRead(FileName,FileNameW))
             continue;
-          ErrHandler.ReadError(FileName);
+          ErrHandler.ReadError(FileName,FileNameW);
         }
     }
     break;
@@ -383,13 +383,13 @@ int File::Read(void *Data,size_t Size)
 // Returns -1 in case of error.
 int File::DirectRead(void *Data,size_t Size)
 {
-#ifdef _WIN_32
+#ifdef _WIN_ALL
   const size_t MaxDeviceRead=20000;
 #endif
 #ifndef _WIN_CE
   if (HandleType==FILE_HANDLESTD)
   {
-#ifdef _WIN_32
+#ifdef _WIN_ALL
     if (Size>MaxDeviceRead)
       Size=MaxDeviceRead;
     hFile=GetStdHandle(STD_INPUT_HANDLE);
@@ -398,7 +398,7 @@ int File::DirectRead(void *Data,size_t Size)
 #endif
   }
 #endif
-#ifdef _WIN_32
+#ifdef _WIN_ALL
   DWORD Read;
   if (!ReadFile(hFile,Data,(DWORD)Size,&Read,NULL))
   {
@@ -427,7 +427,7 @@ int File::DirectRead(void *Data,size_t Size)
 void File::Seek(int64 Offset,int Method)
 {
   if (!RawSeek(Offset,Method) && AllowExceptions)
-    ErrHandler.SeekError(FileName);
+    ErrHandler.SeekError(FileName,FileNameW);
 }
 
 
@@ -440,7 +440,7 @@ bool File::RawSeek(int64 Offset,int Method)
     Offset=(Method==SEEK_CUR ? Tell():FileLength())+Offset;
     Method=SEEK_SET;
   }
-#ifdef _WIN_32
+#ifdef _WIN_ALL
   LONG HighDist=(LONG)(Offset>>32);
   if (SetFilePointer(hFile,(LONG)Offset,&HighDist,Method)==0xffffffff &&
       GetLastError()!=NO_ERROR)
@@ -460,12 +460,12 @@ bool File::RawSeek(int64 Offset,int Method)
 
 int64 File::Tell()
 {
-#ifdef _WIN_32
+#ifdef _WIN_ALL
   LONG HighDist=0;
   uint LowDist=SetFilePointer(hFile,0,&HighDist,FILE_CURRENT);
   if (LowDist==0xffffffff && GetLastError()!=NO_ERROR)
     if (AllowExceptions)
-      ErrHandler.SeekError(FileName);
+      ErrHandler.SeekError(FileName,FileNameW);
     else
       return(-1);
   return(INT32TO64(HighDist,LowDist));
@@ -481,7 +481,7 @@ int64 File::Tell()
 
 void File::Prealloc(int64 Size)
 {
-#ifdef _WIN_32
+#ifdef _WIN_ALL
   if (RawSeek(Size,SEEK_SET))
   {
     Truncate();
@@ -507,7 +507,7 @@ void File::PutByte(byte Byte)
 
 bool File::Truncate()
 {
-#ifdef _WIN_32
+#ifdef _WIN_ALL
   return(SetEndOfFile(hFile)==TRUE);
 #else
   return(false);
@@ -517,7 +517,7 @@ bool File::Truncate()
 
 void File::SetOpenFileTime(RarTime *ftm,RarTime *ftc,RarTime *fta)
 {
-#ifdef _WIN_32
+#ifdef _WIN_ALL
   bool sm=ftm!=NULL && ftm->IsSet();
   bool sc=ftc!=NULL && ftc->IsSet();
   bool sa=fta!=NULL && fta->IsSet();
@@ -548,7 +548,7 @@ void File::SetCloseFileTimeByName(const char *Name,RarTime *ftm,RarTime *fta)
   bool seta=fta!=NULL && fta->IsSet();
   if (setm || seta)
   {
-    struct utimbuf ut;
+    utimbuf ut;
     if (setm)
       ut.modtime=ftm->GetUnix();
     else
@@ -565,7 +565,7 @@ void File::SetCloseFileTimeByName(const char *Name,RarTime *ftm,RarTime *fta)
 
 void File::GetOpenFileTime(RarTime *ft)
 {
-#ifdef _WIN_32
+#ifdef _WIN_ALL
   FILETIME FileTime;
   GetFileTime(hFile,NULL,NULL,&FileTime);
   *ft=FileTime;
@@ -596,7 +596,7 @@ bool File::IsDevice()
 {
   if (hFile==BAD_HANDLE)
     return(false);
-#ifdef _WIN_32
+#ifdef _WIN_ALL
   uint Type=GetFileType(hFile);
   return(Type==FILE_TYPE_CHAR || Type==FILE_TYPE_PIPE);
 #else
@@ -612,7 +612,7 @@ void File::fprintf(const char *fmt,...)
   va_start(argptr,fmt);
   safebuf char Msg[2*NM+1024],OutMsg[2*NM+1024];
   vsprintf(Msg,fmt,argptr);
-#ifdef _WIN_32
+#ifdef _WIN_ALL
   for (int Src=0,Dest=0;;Src++)
   {
     char CurChar=Msg[Src];
